package de.communardo.confluence.plugins.configuration;

import java.util.List;

import org.apache.log4j.Logger;

import com.atlassian.confluence.core.ConfluenceActionSupport;
import com.atlassian.confluence.user.AuthenticatedUserThreadLocal;
import com.atlassian.gzipfilter.org.tuckey.web.filters.urlrewrite.utils.StringUtils;
import com.atlassian.user.User;

import de.communardo.confluence.plugins.addlink.Link;
import de.communardo.confluence.plugins.addlink.v2.LinkCreationException;
import de.communardo.confluence.plugins.addlink.v2.LinkFactory;
import de.communardo.confluence.plugins.addlink.v2.LinkTreeService;
import de.communardo.confluence.plugins.displaysubspace.HierarchyBuilder;
import de.communardo.confluence.plugins.displaysubspace.TreeNodeRenderer;
import de.communardo.confluence.plugins.util.ComponentManager;

/**
 * 
 * @author Bill Martin
 * 
 */
public class ConfigurationAction extends ConfluenceActionSupport {

    /** Logger. */
    private static final Logger LOGGER = Logger.getLogger(ConfigurationAction.class);

    private List<Link> links;
    private boolean globalMenuBarActive;
    private boolean breadcrumbsActive;
    private TreeNodeRenderer treeNodeRenderer;
    private HierarchyBuilder hierarchyBuilder;
    protected SubspaceConfigurationService subspaceConfigurationService;

    private boolean success = false;
    protected LinkTreeService linkTreeService;

    private static final long serialVersionUID = -7644737820456326906L;

    @Override
    public String doDefault() throws Exception {
        setUp();
        setLinks(StaticLinkHelper.loadLinks());
        setGlobalMenuBarActive(subspaceConfigurationService.isGlobalNavigationBarActive());
        setBreadcrumbsActive(subspaceConfigurationService.isBreadcrumbsActive());
        return super.doDefault();
    }

    protected void setUp() {
        User user = AuthenticatedUserThreadLocal.getUser();
        // get the permitted Spaces... possible reason for performance las... use this method
        // carefully
        List permittedSpaces = ComponentManager.getSpaceManager().getPermittedSpaces(user);

        this.setHierarchyBuilder(ConfigureHierarchyHelper.getHierarchyBuilder());
        this.getHierarchyBuilder().init(permittedSpaces);
        this.treeNodeRenderer = new TreeNodeRenderer(null,
                "communardo_plugins/createsubspace/manual_hierarchy/vm/treeNode.vm", null, permittedSpaces);

    }

    public TreeNodeRenderer getTreeNodeRenderer() {
        return treeNodeRenderer;
    }

    public void setTreeNodeRenderer(TreeNodeRenderer treeNodeRenderer) {
        this.treeNodeRenderer = treeNodeRenderer;
    }

    public List<Link> getLinks() {
        return links;
    }

    public List<de.communardo.confluence.plugins.addlink.v2.Link> getLinkTree() {

        List<de.communardo.confluence.plugins.addlink.v2.Link> linkTree = linkTreeService.getLinkTree();

        migrateOldLinks(linkTree);

        return linkTree;
    }

    private void migrateOldLinks(List<de.communardo.confluence.plugins.addlink.v2.Link> linkTree) {
        List<Link> links = StaticLinkHelper.loadLinks();
        if (links != null) {
            for (Link link : links) {
                if (StringUtils.isBlank(link.getId())) {
                    try {
                        de.communardo.confluence.plugins.addlink.v2.Link newLink = LinkFactory.createLink(
                                link.getAlias(), link.getLink(), linkTree);
                        link.setId(newLink.getId());
                        linkTree.add(newLink);
                    } catch (LinkCreationException e) {
                        LOGGER.debug("Can not migrate old link", e);
                    }

                }
            }
        }
        StaticLinkHelper.storeLinks(links);
        linkTreeService.storeLinkTree(linkTree);
    }

    public void setLinks(List<Link> links) {
        this.links = links;
    }

    public boolean isSuccess() {
        return this.success;
    }

    public void setSuccess(boolean val) {
        this.success = val;
    }

    public void setHierarchyBuilder(HierarchyBuilder hierarchyBuilder) {
        this.hierarchyBuilder = hierarchyBuilder;
    }

    public HierarchyBuilder getHierarchyBuilder() {
        return hierarchyBuilder;
    }

    public void setLinkTreeService(LinkTreeService linkTreeService) {
        this.linkTreeService = linkTreeService;
    }

    public void setSubspaceConfigurationService(SubspaceConfigurationService subspaceConfigurationService) {
        this.subspaceConfigurationService = subspaceConfigurationService;
    }

    public boolean isGlobalMenuBarActive() {
        return globalMenuBarActive;
    }

    public void setGlobalMenuBarActive(boolean globalMenuBarActive) {
        this.globalMenuBarActive = globalMenuBarActive;
    }

    public void setBreadcrumbsActive(boolean flag) {
        this.breadcrumbsActive = flag;
    }

    public boolean isBreadcrumbsActive() {
        return breadcrumbsActive;
    }

}
