package de.communardo.confluence.plugins.displaysubspace;

import java.util.Comparator;

import org.apache.commons.lang.StringUtils;

/**
 * Compares nodes for one level in the hierarchy.
 * <p>
 * Refactor me if comparing across multiple levels is required.
 * </p>
 * 
 * @author Bill Martin
 * 
 */
public class TreeNodeComparator implements Comparator<TreeNode> {

    public static final String TITLE = "title";
    public static final String CREATION = "creation";
    public static final String MODIFICATION = "modification";
    public static final String MIXED = "mixed";

    // maybe in the future
    /*
     * public static final String MANUAL = "manual";
     */
    private final String sort;

    /**
     * 
     * @param sort
     */
    public TreeNodeComparator(String sort) {
        this.sort = sort;
    }

    /**
     * Compares two TreeNodes for ordering.
     * 
     * @param node1
     *            to be compared
     * @param node2
     *            to be compared
     * @return a negative integer, zero, or a positive integer
     */
    public int compare(TreeNode node1, TreeNode node2) {
        int cmp;
        if (StringUtils.equalsIgnoreCase(sort, TreeNodeComparator.MODIFICATION)) {
            cmp = compareByModificationDate(node1, node2);
        } else if (StringUtils.equalsIgnoreCase(sort, TreeNodeComparator.TITLE)) {
            cmp = compareByTitle(node1, node2);
        } else if (StringUtils.equalsIgnoreCase(sort, TreeNodeComparator.MIXED)) {
            if (node1.getNodeLevel() == 0) {
                cmp = compareByModificationDate(node1, node2);
            } else {
                cmp = compareByTitle(node1, node2);
            }
        } else {
            cmp = compareByCreationDate(node1, node2);
        }

        // make sure that nodes for two different spaces do not compare equal
        if (cmp == 0 && !node1.getSpace().equals(node2.getSpace())) {
            cmp = 1;
        }
        return cmp;
    }

    /**
     * Compares two TreeNodes for ordering.
     * 
     * @param node1
     *            to be compared
     * @param node2
     *            to be compared
     * @return a negative integer, zero, or a positive integer
     */
    private int compareByTitle(TreeNode node1, TreeNode node2) {
        return node1.getSpace().getName().compareToIgnoreCase(node2.getSpace().getName());
    }

    /**
     * Compares two TreeNodes for ordering.
     * 
     * @param node1
     *            to be compared
     * @param node2
     *            to be compared
     * @return a negative integer, zero, or a positive integer
     */
    private int compareByCreationDate(TreeNode node1, TreeNode node2) {
        return node1.getSpace().getCreationDate().compareTo(node2.getSpace().getCreationDate());
    }

    /**
     * Compares two TreeNodes for ordering.
     * 
     * @param node1
     *            to be compared
     * @param node2
     *            to be compared
     * @return a negative integer, zero, or a positive integer
     */
    private int compareByModificationDate(TreeNode node1, TreeNode node2) {
        return node1.getSpace().getLastModificationDate().compareTo(
                node2.getSpace().getLastModificationDate());
    }
}
