package de.communardo.confluence.plugins.quicksearch;

import java.util.HashSet;
import java.util.Set;

import com.atlassian.confluence.search.service.PredefinedSearchBuilder;
import com.atlassian.confluence.search.service.RecentUpdateQueryParameters;
import com.atlassian.confluence.search.service.SearchQueryParameters;
import com.atlassian.confluence.search.v2.ISearch;
import com.atlassian.confluence.search.v2.Search;
import com.atlassian.confluence.spaces.Space;

import de.communardo.confluence.plugins.displaysubspace.HierarchyBuilder;

public class AdvancedSearchBuilderDelegator implements PredefinedSearchBuilder {

    private final PredefinedSearchBuilder delegateSearchBuilder;
    private final Space topLevelSpace;
    private final HierarchyBuilder hierarchyBuilder;

    public AdvancedSearchBuilderDelegator(PredefinedSearchBuilder defaultPredefinedSearchBuilder,
            HierarchyBuilder hierarchyBuilder, Space topLevelSpace) {

        this.delegateSearchBuilder = defaultPredefinedSearchBuilder;
        this.topLevelSpace = topLevelSpace;
        this.hierarchyBuilder = hierarchyBuilder;
    }

    public ISearch buildRecentUpdateSearch(RecentUpdateQueryParameters recentUpdateQueryParameters,
            int startIndex, int numberOfResults) {
        return delegateSearchBuilder.buildRecentUpdateSearch(recentUpdateQueryParameters,
                startIndex, numberOfResults);
    }

    public ISearch buildSiteSearch(SearchQueryParameters searchQueryParams, int startIndex,
            int pageSize) {

        Set<String> subspacesSpacekeys = new HashSet<String>();
        if (topLevelSpace != null) {

            subspacesSpacekeys.addAll(QuickSearchUtil.getSpacesFromTreeNode(hierarchyBuilder
                    .getNodeFromKey(topLevelSpace.getKey())));
        } else {

            Set<String> spaceKeys = searchQueryParams.getSpaceKeys();
            if (spaceKeys != null) {
                for (String spaceKey : spaceKeys) {

                    subspacesSpacekeys.addAll(QuickSearchUtil
                            .getSpacesFromTreeNode(hierarchyBuilder.getNodeFromKey(spaceKey)));
                }
            }
        }
        searchQueryParams.setSpaceKeys(subspacesSpacekeys);
        return delegateSearchBuilder.buildSiteSearch(searchQueryParams, startIndex, pageSize);
    }

    public ISearch buildUsersSearch(String query, int maxResults) {
        return delegateSearchBuilder.buildUsersSearch(query, maxResults);
    }

    public Search siteSearch(SearchQueryParameters searchQueryParams, int startIndex, int pageSize) {
        return delegateSearchBuilder.siteSearch(searchQueryParams, startIndex, pageSize);
    }

    public Search usersSearch(String query, int maxResults) {
        return delegateSearchBuilder.usersSearch(query, maxResults);
    }

}
