package de.communardo.confluence.plugins.util;

import java.util.Collections;
import java.util.List;

import de.communardo.confluence.plugins.addlink.v2.Link;
import de.communardo.confluence.plugins.addlink.v2.LinkTreeMoveException;

public class LinkTreeHelper {

    public static Link getLink(List<Link> linkList, String id) {

        for (Link link : linkList) {

            if (link.getId().equals(id)) {

                return link;

            } else if (link.getChildren() != null && link.getChildren().size() > 0) {
                Link resultLink = getLink(link.getChildren(), id);
                if (resultLink != null) {
                    return resultLink;
                }

            }

        }

        return null;

    }

    public static boolean editLink(List<Link> linkList, Link newLink) {

        for (Link link : linkList) {

            if (link.getId().equals(newLink.getId())) {

                int index = linkList.indexOf(link);
                linkList.set(index, newLink);
                return true;

            } else if (link.getChildren() != null && link.getChildren().size() > 0) {

                if (editLink(link.getChildren(), newLink)) {
                    return true;
                }

            }

        }

        return false;

    }

    public static boolean deleteLink(List<Link> linkList, String id) {

        for (Link link : linkList) {

            if (link.getId().equals(id)) {

                return linkList.remove(link);

            } else if (link.getChildren() != null && link.getChildren().size() > 0) {

                if (deleteLink(link.getChildren(), id)) {
                    return true;
                }

            }

        }

        return false;

    }

    private enum MoveMode {
        UNINDENT, INDENT, UP, DOWN
    }

    public static void moveLink(List<Link> linkTreeList, String id, String move)
            throws LinkTreeMoveException {
        try {
            move(linkTreeList, null, id, MoveMode.valueOf(move));
        } catch (IllegalArgumentException e) {
            throw new LinkTreeMoveException(e.getMessage(), e);
        } catch (NullPointerException e) {
            throw new LinkTreeMoveException(e.getMessage(), e);
        }
    }

    private static boolean move(List<Link> linkList, List<Link> parentLinkList, String id,
            MoveMode mode) throws LinkTreeMoveException {

        for (Link link : linkList) {

            if (link.getId().equals(id)) {

                return move(linkList, parentLinkList, link, mode);

            } else if (link.getChildren() != null && link.getChildren().size() > 0) {
                if (move(link.getChildren(), linkList, id, mode)) {
                    return true;
                }
            }

        }

        return false;

    }

    private static boolean move(List<Link> linkList, List<Link> parentLinkList, Link link,
            MoveMode mode) throws LinkTreeMoveException {
        int index = linkList.indexOf(link);

        try {

            switch (mode) {
            case UNINDENT:
                if (parentLinkList != null) {
                    parentLinkList.add(link);
                    linkList.remove(link);
                }
                break;
            case INDENT:
                if (index != 0) {
                    linkList.get(--index).getChildren().add(link);
                    linkList.remove(link);
                }
                break;
            case UP:
                if (index != 0) {
                    Collections.swap(linkList, index, --index);
                }
                break;
            case DOWN:
                if (index < linkList.size() - 1) {
                    Collections.swap(linkList, index, ++index);
                }
                break;
            }

        } catch (IndexOutOfBoundsException e) {
            throw new LinkTreeMoveException(e.getMessage(), e);
        }

        return true;
    }
}
