package de.communardo.confluence.plugins.displaysubspace;

import java.util.Date;
import java.util.Iterator;

import junit.framework.Assert;

import org.junit.Test;

import com.atlassian.confluence.spaces.Space;

/**
 * Test HierarchyBuilder
 * 
 * @author Bill Martin
 * @version 1.0
 * 
 */
public class HierarchyBuilderTest {

    /**
     * Test Method "insert"
     */
    @Test()
    public void insert() {

        TreeNodeComparator comparator = new TreeNodeComparator(TreeNodeComparator.TITLE);
        SpaceHierarchy spaceHierarchy = new SpaceHierarchy(comparator);

        Space space1 = new Space("space1");
        space1.setName("ASpace");
        space1.setCreationDate(new Date(1000));
        space1.setLastModificationDate(new Date(2000));

        Space space2 = new Space("space2");
        space2.setName("BSpace");
        space2.setCreationDate(new Date(1));
        space2.setLastModificationDate(new Date(3000));

        Space space3 = new Space("space3");
        space3.setName("CSpace");
        space3.setCreationDate(new Date(100));
        space3.setLastModificationDate(new Date(500));

        Space space4 = new Space("space4");
        space4.setName("BBSpace");
        space4.setCreationDate(new Date(300));
        space4.setLastModificationDate(new Date(400));

        TreeNode node1 = new TreeNode(space1, comparator);
        TreeNode node2 = new TreeNode(space2, comparator);
        TreeNode node3 = new TreeNode(space3, comparator);
        TreeNode node4 = new TreeNode(space4, comparator);

        spaceHierarchy.insertNode(node1, null);
        spaceHierarchy.insertNode(node2, null);
        spaceHierarchy.insertNode(node3, node1);
        spaceHierarchy.insertNode(node4, node1);

        // -------------------------------------------------------------

        Iterator iterator = spaceHierarchy.getHierarchy().iterator();
        TreeNode node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space1.", "space1", node.getSpace().getKey());

        Iterator iter = spaceHierarchy.getHierarchy(node).iterator();
        Assert.assertEquals("The key have to be space4.", "space4", ((TreeNode) iter.next())
                .getSpace().getKey());
        Assert.assertEquals("The key have to be space3.", "space3", ((TreeNode) iter.next())
                .getSpace().getKey());

        node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space2.", "space2", node.getSpace().getKey());

        // -------------------------------------------------------------
        comparator = new TreeNodeComparator(TreeNodeComparator.CREATION);
        SpaceHierarchy spaceHierarchy2 = new SpaceHierarchy(comparator);

        node1 = new TreeNode(space1, comparator);
        node2 = new TreeNode(space2, comparator);
        node3 = new TreeNode(space3, comparator);
        node4 = new TreeNode(space4, comparator);

        spaceHierarchy2.insertNode(node1, null);
        spaceHierarchy2.insertNode(node3, node1);
        spaceHierarchy2.insertNode(node2, null);
        spaceHierarchy2.insertNode(node4, node1);

        iterator = spaceHierarchy2.getHierarchy().iterator();
        node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space2.", "space2", node.getSpace().getKey());
        node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space1.", "space1", node.getSpace().getKey());

        iter = spaceHierarchy2.getHierarchy(node).iterator();
        Assert.assertEquals("The key have to be space3.", "space3", ((TreeNode) iter.next())
                .getSpace().getKey());
        Assert.assertEquals("The key have to be space4.", "space4", ((TreeNode) iter.next())
                .getSpace().getKey());

        // -------------------------------------------------------------
        comparator = new TreeNodeComparator(TreeNodeComparator.MODIFICATION);
        SpaceHierarchy spaceHierarchy3 = new SpaceHierarchy(comparator);

        node1 = new TreeNode(space1, comparator);
        node2 = new TreeNode(space2, comparator);
        node3 = new TreeNode(space3, comparator);
        node4 = new TreeNode(space4, comparator);

        spaceHierarchy3.insertNode(node1, null);
        spaceHierarchy3.insertNode(node3, node1);
        spaceHierarchy3.insertNode(node2, null);
        spaceHierarchy3.insertNode(node4, node1);

        iterator = spaceHierarchy3.getHierarchy().iterator();
        node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space1.", "space1", node.getSpace().getKey());

        iter = spaceHierarchy3.getHierarchy(node).iterator();
        Assert.assertEquals("The key have to be space4.", "space4", ((TreeNode) iter.next())
                .getSpace().getKey());
        Assert.assertEquals("The key have to be space3.", "space3", ((TreeNode) iter.next())
                .getSpace().getKey());

        node = ((TreeNode) iterator.next());
        Assert.assertEquals("The key have to be space2.", "space2", node.getSpace().getKey());

    }
}
